#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_babis
----------------------------------

Tests for `babis` module.
"""
from datetime import datetime

import pytest
from mock import ANY, Mock, call, patch
from six.moves.urllib.error import URLError


from babis import decorator


class TestBabis(object):
    def test_before_and_after(self):
        first_request = Mock()
        second_request = Mock()
        third_request = Mock()

        with patch('babis.babis.Request') as Request:
            Request.side_effect = [
                first_request,
                third_request
            ]
            with patch('babis.babis.urlopen') as urlopen:
                def wrapped():
                    urlopen(second_request)
                decorator(ping_before='foo', ping_after='bar')(wrapped)()

        urlopen.assert_has_calls([
            call(first_request), call(second_request), call(third_request)])

    def test_ping_urls_string(self):
        dec = decorator()
        dec.ping_url = Mock()
        dec.ping_urls('foo')
        dec.ping_url.assert_called_with('foo')

    def test_ping_urls_iterable(self):
        dec = decorator()
        dec.ping_url = Mock()
        dec.ping_urls(('foo', 'bar'))
        dec.ping_url.assert_has_calls([call('foo'), call('bar')])

    def test_ping_url(self):
        dec = decorator()
        with patch('babis.babis.Request') as Request:
            request_init_mock = Mock()
            Request.return_value = request_init_mock
            with patch('babis.babis.urlopen') as urlopen:
                dec.ping_url('foo')
        Request.assert_called_with('foo', data=ANY, headers=ANY)
        urlopen.assert_called_with(request_init_mock)

    def test_user_agent(self):
        dec = decorator(user_agent='foo')
        with patch('babis.babis.Request') as Request:
            with patch('babis.babis.urlopen'):
                dec.ping_url('foo')
        Request.assert_called_with(ANY, data=ANY, headers={'User-Agent': 'foo'})

    def test_method(self):
        dec = decorator(method='post')
        with patch('babis.babis.Request') as Request:
            with patch('babis.babis.urlopen'):
                dec.ping_url('foo')
        Request.assert_called_with(ANY, data='', headers=ANY)

    def test_fail_silenty_false(self):
        dec = decorator(fail_silenty=False)
        with patch('babis.babis.Request'):
            with patch('babis.babis.urlopen') as urlopen:
                urlopen.side_effect = URLError('Boom!')
                with pytest.raises(URLError):
                    dec.ping_url('foo')

    def test_fail_silenty_true(self):
        dec = decorator(fail_silenty=True)
        with patch('babis.babis.Request'):
            with patch('babis.babis.urlopen') as urlopen:
                urlopen.side_effect = URLError('Boom!')
                dec.ping_url('foo')

    def test_rate_parsing(self):
        dec = decorator(rate='5')
        assert(len(dec.calls) == 0)
        assert(dec.ratelimit_count == 5)
        assert(dec.ratelimit_period == 60)

        dec = decorator(rate='5/s')
        assert(dec.ratelimit_period == 1)

        dec = decorator(rate='5/h')
        assert(dec.ratelimit_period == 3600)

        dec = decorator(rate='5/d')
        assert(dec.ratelimit_period == 86400)

        dec = decorator(rate='1/5m')
        assert(dec.ratelimit_period == 300)

        with pytest.raises(Exception):
            dec = decorator(rate='1/5')

    def test_is_ratelimited(self):
        dec = decorator(rate='2/5m')
        with patch('babis.babis.datetime') as datetime_mock:
            datetime_mock.utcnow.side_effect = [
                datetime(2016, 11, 30, 13, 10, 0),
                datetime(2016, 11, 30, 13, 12, 0),
                datetime(2016, 11, 30, 13, 13, 0),
                datetime(2016, 11, 30, 13, 14, 59),
                datetime(2016, 11, 30, 13, 15, 0),
                datetime(2016, 11, 30, 13, 15, 2),
                datetime(2016, 11, 30, 13, 16, 59),
                datetime(2016, 11, 30, 13, 17, 0),
                datetime(2016, 11, 30, 13, 17, 1),
            ]
            assert(dec.is_ratelimited() is False)
            assert(dec.is_ratelimited() is False)
            assert(dec.is_ratelimited() is True)
            assert(dec.is_ratelimited() is True)
            assert(dec.is_ratelimited() is False)
            assert(dec.is_ratelimited() is True)
            assert(dec.is_ratelimited() is True)
            assert(dec.is_ratelimited() is False)
            assert(dec.is_ratelimited() is True)

    def test_ratelimit(self):
        dec = decorator(ping_before='foo', ping_after='bar', rate='2/h')
        dec.ping_urls = Mock()
        wrapped = dec(Mock())
        wrapped()
        wrapped()

        assert(dec.ping_urls.call_count == 2)

    def test_fn_name(self):
        dec = decorator()
        fn = Mock()
        fn.__name__ = 'foobar'
        wrapped = dec(fn)
        assert(wrapped.__name__ == 'foobar')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'six>=1.10'
]

test_requirements = [
    'mock',
    'pytest',
]

setup_requirements = [
    'pytest-runner'
]


setup(
    name='babis',
    version='0.2.2',
    description="Decorator that pings URLs before and after executing the wrapped obj.",
    long_description=readme + '\n\n' + history,
    author="Giorgos Logiotatidis",
    author_email='seadog@sealabs.net',
    url='https://github.com/glogiotatidis/babis',
    packages=[
        'babis',
    ],
    package_dir={'babis':
                 'babis'},
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='babis',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



# -*- coding: utf-8 -*-
import re
from datetime import datetime, timedelta

import six
from six.moves.urllib.request import Request, urlopen
from six.moves.urllib.error import URLError


PERIOD_MAP = {
    's': 1,
    'm': 60,
    'h': 60 * 60,
    'd': 60 * 60 * 24
}


class decorator(object):
    def __init__(self, ping_before=None, ping_after=None, fail_silenty=False,
                 method='GET', user_agent='curl/7.50.1', rate=None):
        self.ping_before = ping_before
        self.ping_after = ping_after
        self.fail_silenty = fail_silenty
        self.method = method
        self.user_agent = user_agent
        if rate:
            self.calls = []
            match = re.match(r'([\d]+)(?:/(\d)?([smhd]))?$', rate)
            if match:
                self.ratelimit_count, multi, ratelimit_period = match.groups()
            else:
                raise Exception('Invalid rate limit')

            try:
                self.ratelimit_count = int(self.ratelimit_count)
            except ValueError:
                raise Exception('Invalid rate limit')

            if not ratelimit_period:
                ratelimit_period = 'm'  # 1 minute
            ratelimit_period = PERIOD_MAP[ratelimit_period]

            if multi:
                ratelimit_period = ratelimit_period * int(multi)

            self.ratelimit_period = ratelimit_period

    def __call__(self, fn):
        def wrapped(*args, **kwargs):
            if self.ping_before and not self.is_ratelimited():
                self.ping_urls(self.ping_before)
            fn(*args, **kwargs)
            if self.ping_after and not self.is_ratelimited():
                self.ping_urls(self.ping_after)
        if hasattr(fn, '__name__'):
            wrapped.__name__ = fn.__name__
        return wrapped

    def is_ratelimited(self):
        if not hasattr(self, 'ratelimit_count'):
            return False

        moment = datetime.utcnow()

        # Clear old calls
        delta = timedelta(seconds=self.ratelimit_period)

        self.calls = [call for call in self.calls if (moment - call) < delta]

        if len(self.calls) == self.ratelimit_count:
            return True

        self.calls.append(moment)
        return False

    def ping_urls(self, urls):
        if isinstance(urls, six.string_types):
            self.ping_url(urls)
        else:
            for url in urls:
                self.ping_url(url)

    def ping_url(self, url):
        method = self.method.lower()
        data = None if method == 'get' else ''
        headers = {
            'User-Agent': self.user_agent,
        }
        request = Request(url, data=data, headers=headers)
        try:
            urlopen(request)
        except URLError:
            if not self.fail_silenty:
                raise
